@@ -47,6 +47,7 @@ static const char *creg_prefix[] = { "+CREG:", NULL };
 static const char *cops_prefix[] = { "+COPS:", NULL };
 static const char *csq_prefix[] = { "+CSQ:", NULL };
 static const char *cind_prefix[] = { "+CIND:", NULL };
+static const char *cmer_prefix[] = { "+CMER:", NULL };
 static const char *zpas_prefix[] = { "+ZPAS:", NULL };
 static const char *option_tech_prefix[] = { "_OCTI:", "_OUWCTI:", NULL };
 
@@ -678,6 +679,54 @@ static void ifx_xhomezr_notify(GAtResult *result, gpointer user_data)
 	ofono_info("Home zone: %s", label);
 }
 
+static void ifx_xreg_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	int state;
+	const char *band;
+	GAtResultIter iter;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+XREG:"))
+		return;
+
+	if (!g_at_result_iter_next_number(&iter, &state))
+		return;
+
+	if (!g_at_result_iter_next_unquoted_string(&iter, &band))
+
+	DBG("state %d band %s", state, band);
+
+	switch (state) {
+	case 0:	/* not registered */
+		nd->tech = -1;
+		break;
+	case 1:	/* registered, GPRS attached */
+		nd->tech = ACCESS_TECHNOLOGY_GSM;
+		break;
+	case 2:	/* registered, EDGE attached */
+		nd->tech = ACCESS_TECHNOLOGY_GSM_EGPRS;
+		break;
+	case 3:	/* registered, WCDMA attached */
+		nd->tech = ACCESS_TECHNOLOGY_UTRAN;
+		break;
+	case 4:	/* registered, HSDPA attached */
+		nd->tech = ACCESS_TECHNOLOGY_UTRAN_HSDPA;
+		break;
+	case 5:	/* registered, HSUPA attached */
+		nd->tech = ACCESS_TECHNOLOGY_UTRAN_HSUPA;
+		break;
+	case 6:	/* registered, HSUPA and HSDPA attached */
+		nd->tech = ACCESS_TECHNOLOGY_UTRAN_HSDPA_HSUPA;
+		break;
+	case 7:	/* registered, GSM */
+		nd->tech = ACCESS_TECHNOLOGY_GSM;
+		break;
+	}
+}
+
 static void ifx_xciev_notify(GAtResult *result, gpointer user_data)
 {
 	//struct ofono_netreg *netreg = user_data;
@@ -758,6 +807,70 @@ static void ciev_notify(GAtResult *result, gpointer user_data)
 	ofono_netreg_strength_notify(netreg, strength);
 }
 
+static void telit_ciev_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	const char *signal_identifier = "rssi";
+	const char *ind_str;
+	int strength;
+	GAtResultIter iter;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+CIEV:"))
+		return;
+
+	if (!g_at_result_iter_next_unquoted_string(&iter, &ind_str))
+		return;
+
+	if (!g_str_equal(signal_identifier, ind_str))
+		return;
+
+	if (!g_at_result_iter_next_number(&iter, &strength))
+		return;
+
+	if (strength == nd->signal_invalid)
+		strength = -1;
+	else
+		strength = (strength * 100) / (nd->signal_max - nd->signal_min);
+
+	ofono_netreg_strength_notify(netreg, strength);
+}
+
+static void cinterion_ciev_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	const char *signal_identifier = "rssi";
+	const char *ind_str;
+	int strength;
+	GAtResultIter iter;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+CIEV:"))
+		return;
+
+	if (!g_at_result_iter_next_unquoted_string(&iter, &ind_str))
+		return;
+
+	if (!g_str_equal(signal_identifier, ind_str))
+		return;
+
+	if (!g_at_result_iter_next_number(&iter, &strength))
+		return;
+
+	DBG("rssi %d", strength);
+
+	if (strength == nd->signal_invalid)
+		strength = -1;
+	else
+		strength = (strength * 100) / (nd->signal_max - nd->signal_min);
+
+	ofono_netreg_strength_notify(netreg, strength);
+}
+
 static void ctzv_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
@@ -780,6 +893,41 @@ static void ctzv_notify(GAtResult *result, gpointer user_data)
 	ofono_netreg_time_notify(netreg, &nd->time);
 }
 
+static void tlts_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	int year, mon, mday, hour, min, sec;
+	char tz[4];
+	const char *time;
+	GAtResultIter iter;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "*TLTS:"))
+		return;
+
+	if (!g_at_result_iter_next_string(&iter, &time))
+		return;
+
+	DBG("time %s", time);
+
+	if (sscanf(time, "%02u/%02u/%02u,%02u:%02u:%02u%s", &year, &mon, &mday,
+						&hour, &min, &sec, tz) != 7)
+                return;
+
+        nd->time.sec = sec;
+        nd->time.min = min;
+        nd->time.hour = hour;
+        nd->time.mday = mday;
+        nd->time.mon = mon;
+        nd->time.year = 2000 + year;
+
+	nd->time.utcoff = atoi(tz) * 15 * 60;
+
+	ofono_netreg_time_notify(netreg, &nd->time);
+}
+
 static gboolean notify_time(gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
@@ -940,6 +1088,27 @@ static void huawei_mode_notify(GAtResult *result, gpointer user_data)
 	}
 }
 
+static void huawei_hcsq_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	GAtResultIter iter;
+	const char *mode;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "^HCSQ:"))
+		return;
+
+	if (!g_at_result_iter_next_string(&iter, &mode))
+		return;
+
+	if (!strcmp("LTE", mode))
+		nd->tech = ACCESS_TECHNOLOGY_EUTRAN;
+
+	/* for other technologies, notification ^MODE is used */
+}
+
 static void huawei_nwtime_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
@@ -1146,6 +1315,57 @@ static void mbm_erinfo_notify(GAtResult *result, gpointer user_data)
 	}
 }
 
+static void icera_nwstate_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	GAtResultIter iter;
+	const char *mccmnc, *tech, *state;
+	int rssi;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (g_at_result_iter_next(&iter, "%NWSTATE:") == FALSE)
+		return;
+
+	if (g_at_result_iter_next_number(&iter, &rssi) == FALSE)
+		return;
+
+	if (g_at_result_iter_next_unquoted_string(&iter, &mccmnc) == FALSE)
+		return;
+
+	if (g_at_result_iter_next_unquoted_string(&iter, &tech) == FALSE)
+		return;
+
+	if (g_at_result_iter_next_unquoted_string(&iter, &state) == FALSE)
+		return;
+
+	DBG("rssi %d tech %s state %s", rssi, tech, state);
+
+	/* small 'g' means CS, big 'G' means PS */
+	if (g_str_equal(tech, "2g") == TRUE ||
+				g_str_equal(tech, "2G") == TRUE ||
+				g_str_equal(tech, "2G-GPRS") == TRUE) {
+		nd->tech = ACCESS_TECHNOLOGY_GSM;
+	} else if (g_str_equal(tech, "2G-EDGE") == TRUE) {
+		nd->tech = ACCESS_TECHNOLOGY_GSM_EGPRS;
+	} else if (g_str_equal(tech, "3g") == TRUE ||
+				g_str_equal(tech, "3G") == TRUE ||
+				g_str_equal(tech, "R99") == TRUE) {
+		if (g_str_equal(state, "HSDPA") == TRUE)
+			nd->tech = ACCESS_TECHNOLOGY_UTRAN_HSDPA;
+		else if (g_str_equal(state, "HSUPA") == TRUE)
+			nd->tech = ACCESS_TECHNOLOGY_UTRAN_HSUPA;
+		else if (g_str_equal(state, "HSDPA-HSUPA") == TRUE)
+			nd->tech = ACCESS_TECHNOLOGY_UTRAN_HSDPA_HSUPA;
+		else if (g_str_equal(state, "HSDPA-HSUPA-HSPA+") == TRUE)
+			nd->tech = ACCESS_TECHNOLOGY_UTRAN_HSDPA_HSUPA;
+		else
+			nd->tech = ACCESS_TECHNOLOGY_UTRAN;
+	} else
+		nd->tech = -1;
+}
+
 static int cnti_to_tech(const char *cnti)
 {
 	if (g_str_equal(cnti, "GSM") == TRUE ||
@@ -1309,6 +1529,187 @@ notify:
 	ofono_netreg_status_notify(netreg, status, lac, ci, tech);
 }
 
+static void at_cmer_not_supported(struct ofono_netreg *netreg)
+{
+	ofono_error("+CMER not supported by this modem.  If this is an error"
+			" please submit patches to support this hardware");
+
+	ofono_netreg_remove(netreg);
+}
+
+static void at_cmer_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+
+	if (!ok) {
+		at_cmer_not_supported(netreg);
+		return;
+	}
+
+	/*
+	 * Telit uses strings instead of numbers to identify indicators
+	 * in a +CIEV URC.
+	 * Handle them in a separate function to keep the code clean.
+	 */
+	if (nd->vendor == OFONO_VENDOR_TELIT)
+		g_at_chat_register(nd->chat, "+CIEV:",
+				telit_ciev_notify, FALSE, netreg, NULL);
+	else
+		g_at_chat_register(nd->chat, "+CIEV:",
+				ciev_notify, FALSE, netreg, NULL);
+
+	g_at_chat_register(nd->chat, "+CREG:",
+				creg_notify, FALSE, netreg, NULL);
+
+	ofono_netreg_register(netreg);
+}
+
+static inline char wanted_cmer(int supported, const char *pref)
+{
+	while (*pref) {
+		if (supported & (1 << (*pref - '0')))
+			return *pref;
+
+		pref++;
+	}
+
+	return '\0';
+}
+
+static inline ofono_bool_t append_cmer_element(char *buf, int *len, int cap,
+						const char *wanted,
+						ofono_bool_t last)
+{
+	char setting = wanted_cmer(cap, wanted);
+
+	if (!setting)
+		return FALSE;
+
+	buf[*len] = setting;
+
+	if (last)
+		buf[*len + 1] = '\0';
+	else
+		buf[*len + 1] = ',';
+
+	*len += 2;
+
+	return TRUE;
+}
+
+static ofono_bool_t build_cmer_string(char *buf, int *cmer_opts,
+					struct netreg_data *nd)
+{
+	const char *ind;
+	int len = sprintf(buf, "AT+CMER=");
+	const char *mode;
+
+	DBG("");
+
+	switch (nd->vendor) {
+	case OFONO_VENDOR_UBLOX_TOBY_L2:
+		/* UBX-13002752 R33: TOBY L2 doesn't support mode 2 and 3 */
+		mode = "1";
+		break;
+	default:
+		mode = "3";
+		break;
+	}
+
+	/*
+	 * Forward unsolicited result codes directly to the TE;
+	 * TA‑TE link specific inband technique used to embed result codes and
+	 * data when TA is in on‑line data mode
+	 */
+	if (!append_cmer_element(buf, &len, cmer_opts[0], mode, FALSE))
+		return FALSE;
+
+	/* No keypad event reporting */
+	if (!append_cmer_element(buf, &len, cmer_opts[1], "0", FALSE))
+		return FALSE;
+
+	/* No display event reporting */
+	if (!append_cmer_element(buf, &len, cmer_opts[2], "0", FALSE))
+		return FALSE;
+
+	switch (nd->vendor) {
+	case OFONO_VENDOR_TELIT:
+		/*
+		 * Telit does not support mode 1.
+		 * All indicator events shall be directed from TA to TE.
+		 */
+		ind = "2";
+		break;
+	default:
+		/*
+		 * Only those indicator events, which are not caused by +CIND
+		 * shall be indicated by the TA to the TE.
+		 */
+		ind = "1";
+		break;
+	}
+
+	/*
+	 * Indicator event reporting using URC +CIEV: <ind>,<value>.
+	 * <ind> indicates the indicator order number (as specified for +CIND)
+	 * and <value> is the new value of indicator.
+	 */
+	if (!append_cmer_element(buf, &len, cmer_opts[3], ind, TRUE))
+		return FALSE;
+
+	return TRUE;
+}
+
+static void at_cmer_query_cb(ofono_bool_t ok, GAtResult *result,
+				gpointer user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	GAtResultIter iter;
+	int cmer_opts_cnt = 5; /* See 27.007 Section 8.10 */
+	int cmer_opts[cmer_opts_cnt];
+	int opt;
+	int mode;
+	char buf[128];
+
+	if (!ok)
+		goto error;
+
+	memset(cmer_opts, 0, sizeof(cmer_opts));
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+CMER:"))
+		goto error;
+
+	for (opt = 0; opt < cmer_opts_cnt; opt++) {
+		int min, max;
+
+		if (!g_at_result_iter_open_list(&iter))
+			goto error;
+
+		while (g_at_result_iter_next_range(&iter, &min, &max)) {
+			for (mode = min; mode <= max; mode++)
+				cmer_opts[opt] |= 1 << mode;
+		}
+
+		if (!g_at_result_iter_close_list(&iter))
+			goto error;
+	}
+
+	if (build_cmer_string(buf, cmer_opts, nd) == FALSE)
+		goto error;
+
+	g_at_chat_send(nd->chat, buf, cmer_prefix,
+			at_cmer_set_cb, netreg, NULL);
+
+	return;
+
+error:
+	at_cmer_not_supported(netreg);
+}
+
 static void cind_support_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
@@ -1320,6 +1721,8 @@ static void cind_support_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	int min = 0;
 	int max = 0;
 	int tmp_min, tmp_max, invalid;
+	int i, len;
+	char buf[256];
 
 	if (!ok)
 		goto error;
@@ -1379,14 +1782,31 @@ static void cind_support_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	if (nd->signal_index == 0)
 		goto error;
 
-	g_at_chat_send(nd->chat, "AT+CMER=3,0,0,1", NULL,
-			NULL, NULL, NULL);
-	g_at_chat_register(nd->chat, "+CIEV:",
-				ciev_notify, FALSE, netreg, NULL);
-	g_at_chat_register(nd->chat, "+CREG:",
-				creg_notify, FALSE, netreg, NULL);
+	/* Turn off all CIEV indicators except the signal indicator */
+	len = sprintf(buf, "AT+CIND=");
+
+	for (i = 1; i < index - 1; i++)
+		len += sprintf(buf + len, i == nd->signal_index ? "1," : "0,");
+
+	len += sprintf(buf + len, i == nd->signal_index ? "1" : "0");
+	g_at_chat_send(nd->chat, buf, NULL, NULL, NULL, NULL);
+
+	switch (nd->vendor) {
+	case OFONO_VENDOR_MBM:
+		/*
+		 * MBM devices report 'CMER: (0,3),(0,2),0,(0-1),0' when
+		 * +CMER=? is executed, which cannot be parsed.  Simply
+		 * send the desired settings in this case.
+		 */
+		g_at_chat_send(nd->chat, "AT+CMER=3,0,0,1", none_prefix,
+				at_cmer_set_cb, netreg, NULL);
+		break;
+	default:
+		g_at_chat_send(nd->chat, "AT+CMER=?", cmer_prefix,
+				at_cmer_query_cb, netreg, NULL);
+		break;
+	}
 
-	ofono_netreg_register(netreg);
 	return;
 
 error:
@@ -1409,6 +1829,14 @@ static void at_creg_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	}
 
 	switch (nd->vendor) {
+	case OFONO_VENDOR_SIMCOM:
+		/* Register for CSQ changes */
+		g_at_chat_send(nd->chat, "AT+AUTOCSQ=1,1", none_prefix,
+				NULL, NULL, NULL);
+
+		g_at_chat_register(nd->chat, "+CSQ:",
+					csq_notify, FALSE, netreg, NULL);
+		break;
 	case OFONO_VENDOR_PHONESIM:
 		g_at_chat_register(nd->chat, "+CSQ:",
 					csq_notify, FALSE, netreg, NULL);
@@ -1489,6 +1917,10 @@ static void at_creg_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		g_at_chat_register(nd->chat, "^MODE:", huawei_mode_notify,
 						FALSE, netreg, NULL);
 
+		/* Register for 4G system mode reports */
+		g_at_chat_register(nd->chat, "^HCSQ:", huawei_hcsq_notify,
+						FALSE, netreg, NULL);
+
 		/* Register for network time reports */
 		g_at_chat_register(nd->chat, "^NWTIME:", huawei_nwtime_notify,
 						FALSE, netreg, NULL);
@@ -1504,6 +1936,16 @@ static void at_creg_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		g_at_chat_send(nd->chat, "AT+XMER=1", none_prefix,
 						NULL, NULL, NULL);
 
+		/* Register for network technology updates */
+		g_at_chat_register(nd->chat, "+XREG:", ifx_xreg_notify,
+						FALSE, netreg, NULL);
+		g_at_chat_send(nd->chat, "AT+XREG=1", none_prefix,
+						NULL, NULL, NULL);
+		g_at_chat_send(nd->chat, "AT+XBANDSEL?", none_prefix,
+						NULL, NULL, NULL);
+		g_at_chat_send(nd->chat, "AT+XUBANDSEL?", none_prefix,
+						NULL, NULL, NULL);
+
 		/* Register for home zone reports */
 		g_at_chat_register(nd->chat, "+XHOMEZR:", ifx_xhomezr_notify,
 						FALSE, netreg, NULL);
@@ -1525,9 +1967,46 @@ static void at_creg_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		g_at_chat_send(nd->chat, "AT+CTZR=1", none_prefix,
 						NULL, NULL, NULL);
 		break;
+	case OFONO_VENDOR_ICERA:
+		/* Register for network technology updates */
+		g_at_chat_register(nd->chat, "%NWSTATE:", icera_nwstate_notify,
+						FALSE, netreg, NULL);
+		g_at_chat_send(nd->chat, "AT%NWSTATE=1", none_prefix,
+						NULL, NULL, NULL);
+
+		/* Register for radio access technology updates */
+		g_at_chat_send(nd->chat, "AT*TRATD=1", none_prefix,
+						NULL, NULL, NULL);
+
+		/* Register for network time update reports */
+		g_at_chat_register(nd->chat, "*TLTS:", tlts_notify,
+						FALSE, netreg, NULL);
+		g_at_chat_send(nd->chat, "AT*TLTS=1", none_prefix,
+						NULL, NULL, NULL);
+		break;
+	case OFONO_VENDOR_CINTERION:
+		/*
+		 * We can't set rssi bounds from Cinterion responses
+		 * so set them up to specified values here
+		 *
+		 * Cinterion rssi signal strength specified as:
+		 * 0      <= -112dBm
+		 * 1 - 4  signal strengh in 15 dB steps
+		 * 5      >= -51 dBm
+		 * 99     not known or undetectable
+		 */
+		nd->signal_min = 0;
+		nd->signal_max = 5;
+		nd->signal_invalid = 99;
+
+		/* Register for specific signal strength reports */
+		g_at_chat_send(nd->chat, "AT^SIND=\"rssi\",1", none_prefix,
+				NULL, NULL, NULL);
+		g_at_chat_register(nd->chat, "+CIEV:",
+				cinterion_ciev_notify, FALSE, netreg, NULL);
+		break;
 	case OFONO_VENDOR_NOKIA:
 	case OFONO_VENDOR_SAMSUNG:
-	case OFONO_VENDOR_SIMCOM:
 		/* Signal strength reporting via CIND is not supported */
 		break;
 	default:
@@ -1555,11 +2034,12 @@ static void at_creg_test_cb(gboolean ok, GAtResult *result, gpointer user_data)
 
 	g_at_result_iter_init(&iter, result);
 
+retry:
 	if (!g_at_result_iter_next(&iter, "+CREG:"))
 		goto error;
 
 	if (!g_at_result_iter_open_list(&iter))
-		goto error;
+		goto retry;
 
 	while (g_at_result_iter_next_range(&iter, &range[0], &range[1])) {
 		if (1 >= range[0] && 1 <= range[1])
