@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
+ *  Copyright (C) 2009-2010  Nokia Corporation and/or its subsidiary(-ies).
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -1504,7 +1504,17 @@ static void isi_dial(struct ofono_voicecall *ovc,
 static void isi_answer(struct ofono_voicecall *ovc, ofono_voicecall_cb_t cb,
 			void *data)
 {
-	isi_call_answer_req(ovc, CALL_ID_ALL, cb, data);
+	struct isi_voicecall *ivc = ofono_voicecall_get_data(ovc);
+	int id;
+
+	for (id = 1; id <= 7; id++)
+		if (ivc->calls[id].status == CALL_STATUS_MT_ALERTING)
+			goto answer_by_id;
+
+	id = CALL_ID_ALL;
+
+answer_by_id:
+	isi_call_answer_req(ovc, id, cb, data);
 }
 
 static void isi_hangup_current(struct ofono_voicecall *ovc,
@@ -1705,7 +1715,7 @@ static void isi_release_specific(struct ofono_voicecall *ovc, int id,
 
 		if ((status->mode_info & CALL_MODE_ORIGINATOR))
 			cause = CALL_CAUSE_BUSY_USER_REQUEST;
-			break;
+		break;
 	}
 
 	isi_call_release_req(ovc, id, CALL_CAUSE_TYPE_CLIENT, cause, cb, data);
@@ -1793,7 +1803,7 @@ static void isi_swap_without_accept(struct ofono_voicecall *ovc,
 static void isi_send_tones(struct ofono_voicecall *ovc, const char *tones,
 				ofono_voicecall_cb_t cb, void *data)
 {
-	isi_call_dtmf_send_req(ovc, CALL_ID_ALL, tones, cb, data);;
+	isi_call_dtmf_send_req(ovc, CALL_ID_ALL, tones, cb, data);
 }
 
 static void subscribe_indications(GIsiClient *cl, void *data)
