@@ -368,15 +368,19 @@ static DBusMessage *mw_set_property(DBusConnection *conn, DBusMessage *msg,
 	return __ofono_error_invalid_args(msg);
 }
 
-static GDBusMethodTable message_waiting_methods[] = {
-	{ "GetProperties",	"",	"a{sv}",	mw_get_properties	},
-	{ "SetProperty",	"sv",	"",		mw_set_property,
-							G_DBUS_METHOD_FLAG_ASYNC },
+static const GDBusMethodTable message_waiting_methods[] = {
+	{ GDBUS_METHOD("GetProperties",
+			NULL, GDBUS_ARGS({ "properties", "a{sv}" }),
+			mw_get_properties) },
+	{ GDBUS_ASYNC_METHOD("SetProperty",
+			GDBUS_ARGS({ "property", "s" }, { "value", "v" }),
+			NULL, mw_set_property) },
 	{ }
 };
 
-static GDBusSignalTable message_waiting_signals[] = {
-	{ "PropertyChanged",	"sv" },
+static const GDBusSignalTable message_waiting_signals[] = {
+	{ GDBUS_SIGNAL("PropertyChanged",
+			GDBUS_ARGS({ "name", "s" }, { "value", "v" })) },
 	{ }
 };
 
@@ -477,7 +481,7 @@ static void mw_mwis_read_cb(int ok, int total_length, int record,
 	status = data[0];
 	data++;
 
-	for (i = 0; i < 5 && i < record_length - 1; i++) {
+	for (i = 0; i < 5 && i < record_length - 1; i++, data++) {
 		info.indication = (status >> i) & 1;
 		info.message_count = info.indication ? data[0] : 0;
 
@@ -618,7 +622,6 @@ static void mw_mbi_read_cb(int ok, int total_length, int record,
 {
 	struct ofono_message_waiting *mw = userdata;
 	int i, err;
-	const unsigned char *st;
 
 	if (!ok || record_length < 4) {
 		ofono_error("Unable to read mailbox identifies "
@@ -652,9 +655,8 @@ out:
 	 * Mailbox numbers located in Byte 1, bits 6 & 5,
 	 * Check for Activated & Allocated
 	 */
-	st = ofono_sim_get_cphs_service_table(mw->sim);
-
-	if (st && bit_field(st[0], 4, 2) == 3) {
+	if (__ofono_sim_cphs_service_available(mw->sim,
+					SIM_CPHS_SERVICE_MAILBOX_NUMBERS)) {
 		ofono_sim_read(mw->sim_context, SIM_EF_CPHS_MBDN_FILEID,
 				OFONO_SIM_FILE_STRUCTURE_FIXED,
 				mw_cphs_mbdn_read_cb, mw);
@@ -730,6 +732,7 @@ static void mw_set_indicator(struct ofono_message_waiting *mw, int profile,
 		efmwis[i + 1] = mw->messages[i].message_count;
 
 	/* Fill in indicator state bits in byte 0 */
+	efmwis[0] = 0;
 	for (i = 0; i < 5 && i < mw->efmwis_length - 1; i++)
 		if (mw->messages[i].indication)
 			efmwis[0] |= 1 << i;
@@ -1024,7 +1027,6 @@ void ofono_message_waiting_register(struct ofono_message_waiting *mw)
 	DBusConnection *conn;
 	const char *path;
 	struct ofono_modem *modem;
-	struct ofono_atom *sim_atom;
 
 	if (mw == NULL)
 		return;
@@ -1045,11 +1047,9 @@ void ofono_message_waiting_register(struct ofono_message_waiting *mw)
 
 	ofono_modem_add_interface(modem, OFONO_MESSAGE_WAITING_INTERFACE);
 
-	sim_atom = __ofono_modem_find_atom(modem, OFONO_ATOM_TYPE_SIM);
-
-	if (sim_atom) {
+	mw->sim = __ofono_atom_find(OFONO_ATOM_TYPE_SIM, modem);
+	if (mw->sim) {
 		/* Assume that if sim atom exists, it is ready */
-		mw->sim = __ofono_atom_get_data(sim_atom);
 		mw->sim_context = ofono_sim_context_create(mw->sim);
 
 		/* Loads MWI states and MBDN from SIM */
