@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -104,6 +104,27 @@ static void timers_update(struct ofono_stk *stk);
 		result.additional_len = sizeof(addn_info);	\
 		result.additional = addn_info;			\
 
+static gboolean convert_to_phone_number_format(const char *input_str,
+							char *output_str)
+{
+	char *digit;
+	char *digit_from = "01234567890abcABC";
+	char *digit_to = "01234567890*#p*#p";
+	int pos;
+
+	for (pos = 0; input_str[pos] != '\0'; pos++) {
+		digit = strchr(digit_from, input_str[pos]);
+		if (digit == NULL)
+			return FALSE;
+
+		output_str[pos] = digit_to[digit - digit_from];
+	}
+
+	output_str[pos] = '\0';
+
+	return TRUE;
+}
+
 static int stk_respond(struct ofono_stk *stk, struct stk_response *rsp,
 			ofono_stk_generic_cb_t cb)
 {
@@ -259,7 +280,7 @@ void __ofono_cbs_sim_download(struct ofono_stk *stk, const struct cbs *msg)
 
 	e.type = STK_ENVELOPE_TYPE_CBS_PP_DOWNLOAD;
 	e.src = STK_DEVICE_IDENTITY_TYPE_NETWORK;
-	memcpy(&e.cbs_pp_download.page, msg, sizeof(msg));
+	memcpy(&e.cbs_pp_download.page, msg, sizeof(*msg));
 
 	err = stk_send_envelope(stk, &e, stk_cbs_download_cb,
 				ENVELOPE_RETRIES_DEFAULT);
@@ -381,6 +402,11 @@ static struct stk_menu *stk_menu_create(const char *title,
 		if (item_icon_ids && item_icon_ids->len)
 			ret->items[i].icon_id = item_icon_ids->list[i];
 
+		if (ret->items[i].icon_id != 0 &&
+				item_icon_ids->qualifier ==
+				STK_ICON_QUALIFIER_TYPE_SELF_EXPLANATORY)
+			ret->items[i].text[0]='\0';
+
 		if (item->id == default_id)
 			ret->default_item = i;
 	}
@@ -476,13 +502,10 @@ static void emit_menu_changed(struct ofono_stk *stk)
 
 static void cancel_pending_dtmf(struct ofono_stk *stk)
 {
-	struct ofono_voicecall *vc = NULL;
-	struct ofono_atom *vc_atom;
+	struct ofono_voicecall *vc;
 
-	vc_atom = __ofono_modem_find_atom(__ofono_atom_get_modem(stk->atom),
-						OFONO_ATOM_TYPE_VOICECALL);
-	if (vc_atom)
-		vc = __ofono_atom_get_data(vc_atom);
+	vc = __ofono_atom_find(OFONO_ATOM_TYPE_VOICECALL,
+				__ofono_atom_get_modem(stk->atom));
 
 	if (vc) /* Should be always true here */
 		__ofono_voicecall_tone_cancel(vc, stk->dtmf_id);
@@ -518,6 +541,21 @@ static gboolean stk_alpha_id_set(struct ofono_stk *stk,
 	if (alpha == NULL)
 		return FALSE;
 
+	if (stk->current_agent == NULL)
+		return FALSE;
+
+	/*
+	 * According to 3GPP TS 102.223 section 8.31:
+	 * If icon is self-explanatory, it replaces the alpha identifier or
+	 * text string.
+	 * If icon is not self-explanatory, it shall be displayed together
+	 * with the alpha identifier or text string.
+	 */
+
+	if (icon->id != 0 && icon->qualifier ==
+			STK_ICON_QUALIFIER_TYPE_SELF_EXPLANATORY)
+		alpha[0]='\0';
+
 	if (stk->respond_on_exit)
 		stk_agent_display_action(stk->current_agent, alpha, icon,
 						user_termination_cb, stk, NULL);
@@ -579,7 +617,7 @@ static DBusMessage *stk_get_properties(DBusConnection *conn,
 					OFONO_PROPERTIES_ARRAY_SIGNATURE,
 					&dict);
 
-	str = stk->idle_mode_text;
+	str = stk->idle_mode_text ? stk->idle_mode_text : "";
 	ofono_dbus_dict_append(&dict, "IdleModeText", DBUS_TYPE_STRING, &str);
 
 	icon = stk->idle_mode_icon.id;
@@ -812,27 +850,34 @@ static DBusMessage *stk_select_item(DBusConnection *conn,
 
 	DBG("");
 
-	if (stk_send_envelope(stk, &e, menu_selection_envelope_cb, 0))
-		return __ofono_error_failed(msg);
-
 	stk->pending = dbus_message_ref(msg);
 
+	if (stk_send_envelope(stk, &e, menu_selection_envelope_cb, 0))
+		__ofono_dbus_pending_reply(&stk->pending,
+					__ofono_error_failed(stk->pending));
+
 	return NULL;
 }
 
-static GDBusMethodTable stk_methods[] = {
-	{ "GetProperties",		"",	"a{sv}",stk_get_properties },
-	{ "SelectItem",			"yo",	"",	stk_select_item,
-					G_DBUS_METHOD_FLAG_ASYNC },
-	{ "RegisterAgent",		"o",	"",	stk_register_agent },
-	{ "UnregisterAgent",		"o",	"",	stk_unregister_agent },
-
+static const GDBusMethodTable stk_methods[] = {
+	{ GDBUS_METHOD("GetProperties",
+			NULL, GDBUS_ARGS({ "properties", "a{sv}" }),
+			stk_get_properties) },
+	{ GDBUS_ASYNC_METHOD("SelectItem",
+			GDBUS_ARGS({ "item", "y" }, { "agent", "o" }), NULL,
+			stk_select_item) },
+	{ GDBUS_METHOD("RegisterAgent",
+			GDBUS_ARGS({ "path", "o" }), NULL,
+			stk_register_agent) },
+	{ GDBUS_METHOD("UnregisterAgent",
+			GDBUS_ARGS({ "path", "o" }), NULL,
+			stk_unregister_agent) },
 	{ }
 };
 
-static GDBusSignalTable stk_signals[] = {
-	{ "PropertyChanged",	"sv" },
-
+static const GDBusSignalTable stk_signals[] = {
+	{ GDBUS_SIGNAL("PropertyChanged",
+			GDBUS_ARGS({ "name", "s" }, { "value", "v" })) },
 	{ }
 };
 
@@ -889,20 +934,17 @@ static gboolean handle_command_send_sms(const struct stk_command *cmd,
 					struct ofono_stk *stk)
 {
 	struct ofono_modem *modem = __ofono_atom_get_modem(stk->atom);
-	struct ofono_atom *sms_atom;
 	struct ofono_sms *sms;
 	GSList msg_list;
 	struct ofono_uuid uuid;
 
-	sms_atom = __ofono_modem_find_atom(modem, OFONO_ATOM_TYPE_SMS);
+	sms = __ofono_atom_find(OFONO_ATOM_TYPE_SMS, modem);
 
-	if (sms_atom == NULL || !__ofono_atom_get_registered(sms_atom)) {
+	if (sms == NULL) {
 		rsp->result.type = STK_RESULT_TYPE_NOT_CAPABLE;
 		return TRUE;
 	}
 
-	sms = __ofono_atom_get_data(sms_atom);
-
 	extern_req_start(stk);
 
 	msg_list.data = (void *) &cmd->send_sms.gsm_sms;
@@ -933,11 +975,11 @@ static gboolean handle_command_set_idle_text(const struct stk_command *cmd,
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
 	const char *path = __ofono_atom_get_path(stk->atom);
+	const struct stk_command_setup_idle_mode_text *sim =
+						&cmd->setup_idle_mode_text;
 	char *idle_mode_text;
 
-	idle_mode_text = dbus_apply_text_attributes(
-					cmd->setup_idle_mode_text.text,
-					&cmd->setup_idle_mode_text.text_attr);
+	idle_mode_text = dbus_apply_text_attributes(sim->text, &sim->text_attr);
 
 	if (idle_mode_text == NULL) {
 		rsp->result.type = STK_RESULT_TYPE_DATA_NOT_UNDERSTOOD;
@@ -947,6 +989,10 @@ static gboolean handle_command_set_idle_text(const struct stk_command *cmd,
 	if (stk->idle_mode_text)
 		g_free(stk->idle_mode_text);
 
+	if (sim->icon_id.id != 0 && sim->icon_id.qualifier ==
+			STK_ICON_QUALIFIER_TYPE_SELF_EXPLANATORY)
+		idle_mode_text[0]='\0';
+
 	stk->idle_mode_text = idle_mode_text;
 
 	ofono_dbus_signal_property_changed(conn, path, OFONO_STK_INTERFACE,
@@ -954,8 +1000,8 @@ static gboolean handle_command_set_idle_text(const struct stk_command *cmd,
 						DBUS_TYPE_STRING,
 						&idle_mode_text);
 
-	if (stk->idle_mode_icon.id != cmd->setup_idle_mode_text.icon_id.id) {
-		memcpy(&stk->idle_mode_icon, &cmd->setup_idle_mode_text.icon_id,
+	if (stk->idle_mode_icon.id != sim->icon_id.id) {
+		memcpy(&stk->idle_mode_icon, &sim->icon_id,
 				sizeof(stk->idle_mode_icon));
 
 		ofono_dbus_signal_property_changed(conn, path,
@@ -1236,7 +1282,7 @@ static gboolean handle_command_select_item(const struct stk_command *cmd,
 					stk->select_item_menu,
 					request_selection_cb, stk,
 					request_selection_destroy,
-					stk->timeout * 1000) < 0) {
+					stk->short_timeout * 1000) < 0) {
 		unsigned char no_cause_result[] = { 0x00 };
 
 		request_selection_destroy(stk);
@@ -1343,11 +1389,18 @@ static gboolean handle_command_display_text(const struct stk_command *cmd,
 	}
 
 	if (qualifier & (1 << 7))
-		timeout = stk->timeout * 1000;
+		timeout = stk->short_timeout * 1000;
 
 	if (dt->duration.interval)
 		timeout = duration_to_msecs(&dt->duration);
 
+	if (cmd->display_text.immediate_response)
+		timeout = stk->timeout * 1000;
+
+	if (dt->icon_id.id != 0 && dt->icon_id.qualifier ==
+			STK_ICON_QUALIFIER_TYPE_SELF_EXPLANATORY)
+		text[0]='\0';
+
 	err = stk_agent_display_text(stk->current_agent, text, &dt->icon_id,
 					priority, display_text_cb, stk,
 					display_text_destroy, timeout);
@@ -1390,8 +1443,10 @@ static void set_get_inkey_duration(struct stk_duration *duration,
 	switch (duration->unit) {
 	case STK_DURATION_TYPE_MINUTES:
 		interval = (interval + 59) / 60;
+		break;
 	case STK_DURATION_TYPE_SECONDS:
 		interval = (interval + 9) / 10;
+		break;
 	case STK_DURATION_TYPE_SECOND_TENTHS:
 		break;
 	}
@@ -1511,17 +1566,15 @@ static gboolean handle_command_get_inkey(const struct stk_command *cmd,
 						struct stk_response *rsp,
 						struct ofono_stk *stk)
 {
-	int timeout = stk->timeout * 1000;
+	int timeout = stk->short_timeout * 1000;
 	const struct stk_command_get_inkey *gi = &cmd->get_inkey;
 	char *text = dbus_apply_text_attributes(gi->text, &gi->text_attr);
 	uint8_t qualifier = stk->pending_cmd->qualifier;
 	gboolean alphabet = (qualifier & (1 << 0)) != 0;
 	gboolean ucs2 = (qualifier & (1 << 1)) != 0;
 	gboolean yesno = (qualifier & (1 << 2)) != 0;
-	/*
-	 * Note: immediate response and help parameter values are not
-	 * provided by current api.
-	 */
+	gboolean immediate = (qualifier & (1 << 3)) != 0;
+	/* Note: help parameter value is not provided by current api. */
 	int err;
 
 	if (text == NULL) {
@@ -1534,6 +1587,10 @@ static gboolean handle_command_get_inkey(const struct stk_command *cmd,
 
 	gettimeofday(&stk->get_inkey_start_ts, NULL);
 
+	if (gi->icon_id.id != 0 && gi->icon_id.qualifier ==
+			STK_ICON_QUALIFIER_TYPE_SELF_EXPLANATORY)
+		text[0]='\0';
+
 	if (yesno)
 		err = stk_agent_request_confirmation(stk->current_agent,
 							text, &gi->icon_id,
@@ -1544,6 +1601,11 @@ static gboolean handle_command_get_inkey(const struct stk_command *cmd,
 						&gi->icon_id, ucs2,
 						request_key_cb, stk, NULL,
 						timeout);
+	else if (immediate)
+		err = stk_agent_request_quick_digit(stk->current_agent,
+							text, &gi->icon_id,
+							request_key_cb, stk,
+							NULL, timeout);
 	else
 		err = stk_agent_request_digit(stk->current_agent, text,
 						&gi->icon_id, request_key_cb,
@@ -1624,6 +1686,10 @@ static gboolean handle_command_get_input(const struct stk_command *cmd,
 		return TRUE;
 	}
 
+	if (gi->icon_id.id != 0 && gi->icon_id.qualifier ==
+			STK_ICON_QUALIFIER_TYPE_SELF_EXPLANATORY)
+		text[0]='\0';
+
 	if (alphabet)
 		err = stk_agent_request_input(stk->current_agent, text,
 						&gi->icon_id, gi->default_text,
@@ -1688,14 +1754,10 @@ static void call_setup_connected(struct ofono_call *call, void *data)
 static void call_setup_cancel(struct ofono_stk *stk)
 {
 	struct ofono_voicecall *vc;
-	struct ofono_atom *vc_atom;
 
-	vc_atom = __ofono_modem_find_atom(__ofono_atom_get_modem(stk->atom),
-						OFONO_ATOM_TYPE_VOICECALL);
-	if (vc_atom == NULL)
-		return;
+	vc = __ofono_atom_find(OFONO_ATOM_TYPE_VOICECALL,
+				__ofono_atom_get_modem(stk->atom));
 
-	vc = __ofono_atom_get_data(vc_atom);
 	if (vc)
 		__ofono_voicecall_dial_cancel(vc);
 }
@@ -1710,9 +1772,10 @@ static void confirm_call_cb(enum stk_agent_result result, gboolean confirm,
 	static unsigned char busy_on_call_result[] = { 0x02 };
 	static unsigned char no_cause_result[] = { 0x00 };
 	char *alpha_id = NULL;
-	struct ofono_voicecall *vc = NULL;
-	struct ofono_atom *vc_atom;
+	struct ofono_voicecall *vc;
 	struct stk_response rsp;
+	char number[256];
+	char *pause_chr;
 	int err;
 
 	switch (result) {
@@ -1733,11 +1796,8 @@ static void confirm_call_cb(enum stk_agent_result result, gboolean confirm,
 		return;
 	}
 
-	vc_atom = __ofono_modem_find_atom(__ofono_atom_get_modem(stk->atom),
-						OFONO_ATOM_TYPE_VOICECALL);
-	if (vc_atom)
-		vc = __ofono_atom_get_data(vc_atom);
-
+	vc = __ofono_atom_find(OFONO_ATOM_TYPE_VOICECALL,
+				__ofono_atom_get_modem(stk->atom));
 	if (vc == NULL) {
 		send_simple_response(stk, STK_RESULT_TYPE_NOT_CAPABLE);
 		return;
@@ -1753,7 +1813,20 @@ static void confirm_call_cb(enum stk_agent_result result, gboolean confirm,
 		}
 	}
 
-	err = __ofono_voicecall_dial(vc, sc->addr.number, sc->addr.ton_npi,
+	/* Convert the setup call number to phone number format */
+	if (convert_to_phone_number_format(sc->addr.number, number) == FALSE) {
+		send_simple_response(stk, STK_RESULT_TYPE_DATA_NOT_UNDERSTOOD);
+		return;
+	}
+
+	/* Remove the DTMF string from the phone number */
+	pause_chr = strchr(number, 'p');
+
+	if (pause_chr)
+		number[pause_chr - number] = '\0';
+
+	/* TODO: send the DTMF after call is connected */
+	err = __ofono_voicecall_dial(vc, number, sc->addr.ton_npi,
 					alpha_id, sc->icon_id_call_setup.id,
 					qualifier >> 1, call_setup_connected,
 					stk);
@@ -1798,8 +1871,9 @@ static void confirm_handled_call_cb(enum stk_agent_result result,
 	struct ofono_stk *stk = user_data;
 	const struct stk_command_setup_call *sc =
 					&stk->pending_cmd->setup_call;
-	struct ofono_voicecall *vc = NULL;
-	struct ofono_atom *vc_atom;
+	struct ofono_voicecall *vc;
+	char number[256];
+	char *pause_chr;
 
 	if (stk->driver->user_confirmation == NULL)
 		goto out;
@@ -1809,17 +1883,25 @@ static void confirm_handled_call_cb(enum stk_agent_result result,
 		goto out;
 	}
 
+	if (convert_to_phone_number_format(sc->addr.number, number) == FALSE) {
+		stk->driver->user_confirmation(stk, FALSE);
+		goto out;
+	}
+
+	/* Remove the DTMF string from the phone number */
+	pause_chr = strchr(number, 'p');
+
+	if (pause_chr)
+		number[pause_chr - number] = '\0';
+
 	stk->driver->user_confirmation(stk, confirm);
 
-	vc_atom = __ofono_modem_find_atom(__ofono_atom_get_modem(stk->atom),
-						OFONO_ATOM_TYPE_VOICECALL);
-	if (vc_atom)
-		vc = __ofono_atom_get_data(vc_atom);
-
+	vc = __ofono_atom_find(OFONO_ATOM_TYPE_VOICECALL,
+				__ofono_atom_get_modem(stk->atom));
 	if (vc == NULL)
 		goto out;
 
-	__ofono_voicecall_set_alpha_and_icon_id(vc, sc->addr.number,
+	__ofono_voicecall_set_alpha_and_icon_id(vc, number,
 						sc->addr.ton_npi,
 						sc->alpha_id_call_setup,
 						sc->icon_id_call_setup.id);
@@ -1840,7 +1922,6 @@ static gboolean handle_command_set_up_call(const struct stk_command *cmd,
 	static unsigned char busy_on_call_result[] = { 0x02 };
 	char *alpha_id = NULL;
 	struct ofono_voicecall *vc = NULL;
-	struct ofono_atom *vc_atom;
 	int err;
 
 	if (qualifier > 5) {
@@ -1857,11 +1938,8 @@ static gboolean handle_command_set_up_call(const struct stk_command *cmd,
 		return TRUE;
 	}
 
-	vc_atom = __ofono_modem_find_atom(__ofono_atom_get_modem(stk->atom),
-						OFONO_ATOM_TYPE_VOICECALL);
-	if (vc_atom)
-		vc = __ofono_atom_get_data(vc_atom);
-
+	vc = __ofono_atom_find(OFONO_ATOM_TYPE_VOICECALL,
+				__ofono_atom_get_modem(stk->atom));
 	if (vc == NULL) {
 		rsp->result.type = STK_RESULT_TYPE_NOT_CAPABLE;
 		return TRUE;
@@ -1907,14 +1985,9 @@ static gboolean handle_command_set_up_call(const struct stk_command *cmd,
 static void send_ussd_cancel(struct ofono_stk *stk)
 {
 	struct ofono_ussd *ussd;
-	struct ofono_atom *atom;
 
-	atom = __ofono_modem_find_atom(__ofono_atom_get_modem(stk->atom),
-						OFONO_ATOM_TYPE_USSD);
-	if (atom == NULL)
-		return;
-
-	ussd = __ofono_atom_get_data(atom);
+	ussd = __ofono_atom_find(OFONO_ATOM_TYPE_USSD,
+					__ofono_atom_get_modem(stk->atom));
 	if (ussd)
 		__ofono_ussd_initiate_cancel(ussd);
 
@@ -2015,17 +2088,15 @@ static gboolean handle_command_send_ussd(const struct stk_command *cmd,
 	struct ofono_modem *modem = __ofono_atom_get_modem(stk->atom);
 	static unsigned char busy_on_ss_result[] = { 0x03 };
 	static unsigned char busy_on_ussd_result[] = { 0x08 };
-	struct ofono_atom *atom;
 	struct ofono_ussd *ussd;
 	int err;
 
-	atom = __ofono_modem_find_atom(modem, OFONO_ATOM_TYPE_USSD);
-	if (atom == NULL || !__ofono_atom_get_registered(atom)) {
+	ussd = __ofono_atom_find(OFONO_ATOM_TYPE_USSD, modem);
+	if (ussd == NULL) {
 		rsp->result.type = STK_RESULT_TYPE_NOT_CAPABLE;
 		return TRUE;
 	}
 
-	ussd = __ofono_atom_get_data(atom);
 	if (__ofono_ussd_is_busy(ussd)) {
 		ADD_ERROR_RESULT(rsp->result, STK_RESULT_TYPE_TERMINAL_BUSY,
 					busy_on_ussd_result);
@@ -2043,11 +2114,6 @@ static gboolean handle_command_send_ussd(const struct stk_command *cmd,
 					cmd->send_ussd.ussd_string.len,
 					send_ussd_callback, stk);
 
-	if (err >= 0) {
-		stk->cancel_cmd = send_ussd_cancel;
-
-		return FALSE;
-	}
 
 	if (err == -ENOSYS) {
 		rsp->result.type = STK_RESULT_TYPE_NOT_CAPABLE;
@@ -2060,6 +2126,20 @@ static gboolean handle_command_send_ussd(const struct stk_command *cmd,
 		return TRUE;
 	}
 
+	if (err < 0) {
+		unsigned char no_cause_result[] = { 0x00 };
+
+		/*
+		 * We most likely got an out of memory error, tell SIM
+		 * to retry
+		 */
+		ADD_ERROR_RESULT(rsp->result, STK_RESULT_TYPE_TERMINAL_BUSY,
+					no_cause_result);
+		return TRUE;
+	}
+
+	stk->cancel_cmd = send_ussd_cancel;
+
 	stk_alpha_id_set(stk, cmd->send_ussd.alpha_id,
 				&cmd->send_ussd.text_attr,
 				&cmd->send_ussd.icon_id);
@@ -2081,12 +2161,8 @@ static gboolean handle_command_refresh(const struct stk_command *cmd,
 					struct ofono_stk *stk)
 {
 	struct ofono_error failure = { .type = OFONO_ERROR_TYPE_FAILURE };
-	struct ofono_sim *sim = NULL;
-	struct ofono_atom *sim_atom;
-	struct ofono_ussd *ussd = NULL;
-	struct ofono_atom *ussd_atom;
-	struct ofono_voicecall *vc = NULL;
-	struct ofono_atom *vc_atom;
+	struct ofono_modem *modem = __ofono_atom_get_modem(stk->atom);
+	struct ofono_sim *sim;
 	uint8_t addnl_info[1];
 	int err;
 	GSList *l;
@@ -2145,11 +2221,7 @@ static gboolean handle_command_refresh(const struct stk_command *cmd,
 					cmd->refresh.icon_id.qualifier);
 	DBG("Alpha ID: %s", cmd->refresh.alpha_id);
 
-	sim_atom = __ofono_modem_find_atom(__ofono_atom_get_modem(stk->atom),
-						OFONO_ATOM_TYPE_SIM);
-	if (sim_atom)
-		sim = __ofono_atom_get_data(sim_atom);
-
+	sim = __ofono_atom_find(OFONO_ATOM_TYPE_SIM, modem);
 	if (sim == NULL) {
 		if (rsp != NULL)
 			rsp->result.type = STK_RESULT_TYPE_NOT_CAPABLE;
@@ -2158,11 +2230,10 @@ static gboolean handle_command_refresh(const struct stk_command *cmd,
 	}
 
 	if (rsp != NULL) {
-		ussd_atom = __ofono_modem_find_atom(
-					__ofono_atom_get_modem(stk->atom),
-					OFONO_ATOM_TYPE_USSD);
-		if (ussd_atom)
-			ussd = __ofono_atom_get_data(ussd_atom);
+		struct ofono_ussd *ussd;
+		struct ofono_voicecall *vc;
+
+		ussd = __ofono_atom_find(OFONO_ATOM_TYPE_USSD, modem);
 
 		if (ussd && __ofono_ussd_is_busy(ussd)) {
 			addnl_info[0] = STK_RESULT_ADDNL_ME_PB_USSD_BUSY;
@@ -2173,11 +2244,7 @@ static gboolean handle_command_refresh(const struct stk_command *cmd,
 			return TRUE;
 		}
 
-		vc_atom = __ofono_modem_find_atom(
-					__ofono_atom_get_modem(stk->atom),
-					OFONO_ATOM_TYPE_VOICECALL);
-		if (vc_atom)
-			vc = __ofono_atom_get_data(vc_atom);
+		vc = __ofono_atom_find(OFONO_ATOM_TYPE_VOICECALL, modem);
 
 		if (vc && __ofono_voicecall_is_busy(vc,
 					OFONO_VOICECALL_INTERACTION_NONE)) {
@@ -2250,8 +2317,7 @@ static gboolean handle_command_refresh(const struct stk_command *cmd,
 			break;
 		}
 
-		g_slist_foreach(file_list, (GFunc) g_free, NULL);
-		g_slist_free(file_list);
+		g_slist_free_full(file_list, g_free);
 
 		return FALSE;
 	}
@@ -2313,22 +2379,64 @@ static gboolean handle_command_provide_local_info(const struct stk_command *cmd,
 				struct stk_response *rsp, struct ofono_stk *stk)
 {
 	switch (cmd->qualifier) {
+	case 0:
+		DBG("Unhandled - Local information according to current NAA");
+		break;
+	case 1:
+		DBG("Unhandled - IMEI");
+		break;
+	case 2:
+		DBG("Unhandled - Network measurement results");
+		break;
 	case 3:
 		DBG("Date, time and time zone");
 		get_time(rsp);
 		return TRUE;
-
 	case 4:
 		DBG("Language setting");
 		get_lang(rsp, stk);
 		return FALSE;
+	case 6:
+		DBG("Unhandled - Access Technology");
+		break;
+	case 7:
+		DBG("Unhandled - ESN of the terminal");
+		break;
+	case 8:
+		DBG("Unhandled - IMEISV of the terminal");
+		break;
+	case 9:
+		DBG("Unhandled - Search mode");
+		break;
+	case 10:
+		DBG("Unhandled - Charge state of Battery");
+		break;
+	case 11:
+		DBG("Unhandled - MEID");
+		break;
+	case 13:
+		DBG("Unhandled - Broadcast Network information");
+		break;
+	case 14:
+		DBG("Unhandled - Multiple Access Technologies");
+		break;
+	case 15:
+		DBG("Unhandled - Location information for multiple access"
+			" technologies");
+		break;
+	case 16:
+		DBG("Unhandled - Measurement results for multiple access"
+			" technologies");
+		break;
 
 	default:
 		ofono_info("Unsupported Provide Local Info qualifier: %d",
 				cmd->qualifier);
-		rsp->result.type = STK_RESULT_TYPE_NOT_CAPABLE;
-		return TRUE;
+		break;
 	}
+
+	rsp->result.type = STK_RESULT_TYPE_NOT_CAPABLE;
+	return TRUE;
 }
 
 static void send_dtmf_cancel(struct ofono_stk *stk)
@@ -2373,35 +2481,23 @@ static gboolean handle_command_send_dtmf(const struct stk_command *cmd,
 {
 	static unsigned char not_in_speech_call_result[] = { 0x07 };
 	struct ofono_voicecall *vc = NULL;
-	struct ofono_atom *vc_atom;
-	char dtmf[256], *digit;
-	char *dtmf_from = "01234567890abcABC";
-	char *dtmf_to = "01234567890*#p*#p";
-	int err, pos;
-
-	vc_atom = __ofono_modem_find_atom(__ofono_atom_get_modem(stk->atom),
-						OFONO_ATOM_TYPE_VOICECALL);
-	if (vc_atom)
-		vc = __ofono_atom_get_data(vc_atom);
+	char dtmf[256];
+	int err;
 
+	vc = __ofono_atom_find(OFONO_ATOM_TYPE_VOICECALL,
+				__ofono_atom_get_modem(stk->atom));
 	if (vc == NULL) {
 		rsp->result.type = STK_RESULT_TYPE_NOT_CAPABLE;
 		return TRUE;
 	}
 
 	/* Convert the DTMF string to phone number format */
-	for (pos = 0; cmd->send_dtmf.dtmf[pos] != '\0'; pos++) {
-		digit = strchr(dtmf_from, cmd->send_dtmf.dtmf[pos]);
-		if (digit == NULL) {
-			rsp->result.type = STK_RESULT_TYPE_DATA_NOT_UNDERSTOOD;
-			return TRUE;
-		}
-
-		dtmf[pos] = dtmf_to[digit - dtmf_from];
+	if (convert_to_phone_number_format(cmd->send_dtmf.dtmf, dtmf) ==
+						FALSE) {
+		rsp->result.type = STK_RESULT_TYPE_DATA_NOT_UNDERSTOOD;
+		return TRUE;
 	}
 
-	dtmf[pos] = '\0';
-
 	err = __ofono_voicecall_tone_send(vc, dtmf, dtmf_sent_cb, stk);
 
 	if (err == -ENOSYS) {
@@ -2472,7 +2568,7 @@ static gboolean handle_command_play_tone(const struct stk_command *cmd,
 	static const struct {
 		const char *name;
 		/* Continuous true/false according to 02.40 */
-		gboolean continuous;
+		gboolean repeatable;
 	} tone_infos[] = {
 		/* Default */
 		[0x00] = { "general-beep", FALSE },
@@ -2482,9 +2578,9 @@ static gboolean handle_command_play_tone(const struct stk_command *cmd,
 		[0x02] = { "busy", TRUE },
 		[0x03] = { "congestion", TRUE },
 		[0x04] = { "radio-path-acknowledge", FALSE },
-		[0x05] = { "radio-path-not-available", FALSE },
+		[0x05] = { "radio-path-not-available", TRUE },
 		[0x06] = { "error", TRUE },
-		[0x07] = { "call-waiting", FALSE },
+		[0x07] = { "call-waiting", TRUE },
 		[0x08] = { "ringing-tone", TRUE },
 
 		/* Proprietary */
@@ -2541,7 +2637,19 @@ static gboolean handle_command_play_tone(const struct stk_command *cmd,
 	else
 		timeout = manufacturer_timeout;
 
-	if (!tone_infos[pt->tone].continuous)
+	if (pt->icon_id.id != 0 && pt->icon_id.qualifier ==
+			STK_ICON_QUALIFIER_TYPE_SELF_EXPLANATORY)
+		text[0]='\0';
+
+	/*
+	 * According to TS 102.223 section 6.6.6:
+	 * "the length of time for which the Terminal shall generate the tone,
+	 * if the tone is contunious or repeatable.  For single tones, the
+	 * value of this data object shall be ignored by the Terminal.  If no
+	 * duration is specified, the Terminal shall default to a duration
+	 * determined by the Terminal manufacturer
+	 */
+	if (!tone_infos[pt->tone].repeatable)
 		/* Duration ignored */
 		err = stk_agent_play_tone(stk->current_agent, text,
 						&pt->icon_id, vibrate,
@@ -2647,42 +2755,15 @@ static gboolean handle_command_launch_browser(const struct stk_command *cmd,
 	return FALSE;
 }
 
-static void proactive_command_handled_end(struct ofono_stk *stk)
+static void setup_call_handled_cancel(struct ofono_stk *stk)
 {
-	if (stk->pending_cmd == NULL)
-		return;
+	struct ofono_voicecall *vc;
 
-	switch(stk->pending_cmd->type) {
-	case STK_COMMAND_TYPE_SETUP_CALL:
-	{
-		struct ofono_voicecall *vc = NULL;
-		struct ofono_atom *vc_atom;
+	vc = __ofono_atom_find(OFONO_ATOM_TYPE_VOICECALL,
+				__ofono_atom_get_modem(stk->atom));
 
-		vc_atom = __ofono_modem_find_atom(
-					__ofono_atom_get_modem(stk->atom),
-					OFONO_ATOM_TYPE_VOICECALL);
-		if (vc_atom)
-			vc = __ofono_atom_get_data(vc_atom);
-
-		if (vc != NULL)
-			__ofono_voicecall_clear_alpha_and_icon_id(vc);
-
-		break;
-	}
-	case STK_COMMAND_TYPE_SEND_SMS:
-	case STK_COMMAND_TYPE_SEND_USSD:
-	case STK_COMMAND_TYPE_SEND_SS:
-	case STK_COMMAND_TYPE_SEND_DTMF:
-		stk_alpha_id_unset(stk);
-		break;
-
-	default:
-		break;
-	}
-
-	stk_command_free(stk->pending_cmd);
-	stk->pending_cmd = NULL;
-	stk->cancel_cmd = NULL;
+	if (vc != NULL)
+		__ofono_voicecall_clear_alpha_and_icon_id(vc);
 }
 
 static gboolean handle_setup_call_confirmation_req(struct stk_command *cmd,
@@ -2697,6 +2778,9 @@ static gboolean handle_setup_call_confirmation_req(struct stk_command *cmd,
 	if (alpha_id == NULL)
 		goto out;
 
+	if (stk->current_agent == FALSE)
+		goto out;
+
 	err = stk_agent_confirm_call(stk->current_agent, alpha_id,
 					&sc->icon_id_usr_cfm,
 					confirm_handled_call_cb,
@@ -2707,7 +2791,7 @@ static gboolean handle_setup_call_confirmation_req(struct stk_command *cmd,
 	if (err < 0)
 		goto out;
 
-	stk->cancel_cmd = proactive_command_handled_end;
+	stk->cancel_cmd = setup_call_handled_cancel;
 
 	return TRUE;
 
@@ -2804,6 +2888,9 @@ void ofono_stk_proactive_command_notify(struct ofono_stk *stk,
 		case STK_COMMAND_TYPE_GET_INPUT:
 		case STK_COMMAND_TYPE_PLAY_TONE:
 		case STK_COMMAND_TYPE_SETUP_CALL:
+		case STK_COMMAND_TYPE_SEND_SMS:
+		case STK_COMMAND_TYPE_SEND_USSD:
+		case STK_COMMAND_TYPE_SEND_DTMF:
 			send_simple_response(stk, STK_RESULT_TYPE_NOT_CAPABLE);
 			return;
 
@@ -2931,7 +3018,7 @@ static gboolean handled_alpha_id_set(struct ofono_stk *stk,
 	if (stk_alpha_id_set(stk, text, attr, icon) == FALSE)
 		return FALSE;
 
-	stk->cancel_cmd = proactive_command_handled_end;
+	stk->cancel_cmd = stk_alpha_id_unset;
 	return TRUE;
 }
 
@@ -2949,7 +3036,7 @@ void ofono_stk_proactive_command_handled_notify(struct ofono_stk *stk,
 	 * responses here
 	 */
 	if (length > 0 && pdu[0] == 0x81) {
-		proactive_command_handled_end(stk);
+		stk_proactive_command_cancel(stk);
 		return;
 	}
 
@@ -3009,8 +3096,14 @@ void ofono_stk_proactive_command_handled_notify(struct ofono_stk *stk,
 		break;
 
 	case STK_COMMAND_TYPE_REFRESH:
+		/*
+		 * On a refresh we should not try to free the pending command,
+		 * as the stk atom itself likely disappeared as a result.
+		 * If it has not, then any subsequent proactive command, or
+		 * session end notification will free it anyway
+		 */
 		handle_command_refresh(stk->pending_cmd, NULL, stk);
-		break;
+		return;
 	}
 
 out:
@@ -3071,8 +3164,7 @@ static void stk_unregister(struct ofono_atom *atom)
 		stk->main_menu = NULL;
 	}
 
-	g_queue_foreach(stk->envelope_q, (GFunc) g_free, NULL);
-	g_queue_free(stk->envelope_q);
+	g_queue_free_full(stk->envelope_q, g_free);
 
 	ofono_modem_remove_interface(modem, OFONO_STK_INTERFACE);
 	g_dbus_unregister_interface(conn, path, OFONO_STK_INTERFACE);
@@ -3148,7 +3240,7 @@ void ofono_stk_register(struct ofono_stk *stk)
 	__ofono_atom_register(stk->atom, stk_unregister);
 
 	stk->timeout = 180; /* 3 minutes */
-	stk->short_timeout = 20; /* 20 seconds */
+	stk->short_timeout = 25; /* 25 seconds */
 	stk->envelope_q = g_queue_new();
 }
 
